
#include <allegro.h>
#include <iostream>
#include <vector>
#include <math.h>
#include <list>
#include <set>
#include <fstream>
#include <algorithm>
#ifdef __APPLE__
#include <ApplicationServices/ApplicationServices.h>
#endif
#include "random.hpp"

using namespace std;

BITMAP* buffer;

int magenta;
int red;
int lightred;
int black;
int white;
int green;
int lightgreen;
int orange;
int purp0;
int purp1;
int purp2;
int grey;
int lightgrey;
int blue;
int yellow;
int bgcol;

bool quit=false;


int algdone=false;

char leveltitle[200];


int levelseed;
const int SCREEN_WIDTH=320;
const int SCREEN_HEIGHT=240;
const int MARGIN=10;
const int TRIANGLE_SIZE = 50;
const int ICON_RADIUS = 10;
const int FRAMERATE=30;
const int FONT_HEIGHT=8;
int bltime;
int endmenutimer;
bool endmode;

bool gameover;
uint musictimer;

bool dragmode;
uint curnote;

uint melody_length=10;
int level_code;

struct Pos{
	int x;
	int y;
};

//order in which to visit melodies
vector<int> divineOrdering;

char orderchars[30] = {'A','B','G','H','C','N','M','S','T','O','I','D','J','E','P','U','[','Z','Y','K','F','Q','V','\\','L','W','R',']','^','X'};

Pos clickPos;


inline void shuffle(vector<int>& v)
{
	int n=v.size();
	while(n>1)
	{
		n--;
		int k = get_rand()%(n+1);
		swap(v[k],v[n]);
	}

}

vector<int> intersectVectors(vector<int> a, vector<int> b)
{
	vector<int> v;
	for (uint i=0;i<a.size();i++)
	{
		for (uint j=0;j<b.size();j++)
		{
			if (a[i]==b[j])
			{
				v.push_back(a[i]);
				break;
			}
		}
	}
	return v;
}

vector<int> intersectVectors(vector<vector<int> > vectors)
{
	if (vectors.size()==0)
	{
		return vector<int>();
	}

	vector<int> v = vectors[0];
	for(uint i=1;i<vectors.size();i++)
	{
		v=intersectVectors(v,vectors[i]);
	}

	return v;
}


inline float distsq(const Pos& a, const Pos& b)
{
	return ((b.x-a.x)*(b.x-a.x)+(b.y-a.y)*(b.y-a.y));
}

Pos highlight;

Pos mouse;
bool mb1=false;
bool mb2=false;

typedef vector<int> Melody;


struct backtrackState
{
	vector<Melody> melodies;
	backtrackState():melodies(vector<Melody>())
	{
		melodies.clear();
		for (uint i=0;i<200;i++)
			melodies.push_back(vector<int>() );
	}
};

backtrackState gameMelodies_State;

struct M_Line {
	int melody;
	int voice; //either 0, 1 or 2
};

vector<M_Line> m_lines;

struct Triangle {
	M_Line* line[3];
	vector<bool> NoteBad;
	bool anybad;
	Triangle(): NoteBad(vector<bool>()), anybad(true)
	{

	}
};

const uint MINTERVALCOUNT=16;
const int m_intervals[MINTERVALCOUNT]={0,5,-5,7,-7,12,-12,1,-1,2,-2,3,-3,4,-4,8};

//perfect
const uint HPINTERVALCOUNT=7;
const int h_pintervals[HPINTERVALCOUNT]={0,7,12,19,24,31,36};

//imperfect
const uint HIPINTERVALCOUNT=12;
const int h_ipintervals[HIPINTERVALCOUNT]={3,4,8,9 ,12+3,12+4,12+8,12+9,24+3,24+4,24+8,24+9};

Melody absolutify(M_Line* a,const backtrackState& s)
{
	Melody m;
	assert(a);
	assert(s.melodies.size()>a->melody);
	if(s.melodies[a->melody].size()==0)
	{//see if i missed a character

		bool found=false;;
		for (uint i=0;i<30;i++)
		{
			if (orderchars[i]-'A'==a->melody)
			{
				found=true;
				break;
			}
		}
		if (!found)
			cout<<"not found char "<<a->melody<<endl;
		assert(found);
		assert(false);

	}
	for (uint i=0;i<s.melodies[a->melody].size();i++)
	{
		m.push_back (s.melodies[a->melody][i]+(a->voice-2)*12);
	}
	return m;
}



//says if last steps provided constitute valid progression
bool allowedProgression(M_Line* la, M_Line* lb,const backtrackState& s)
{

	uint minsize=min(s.melodies[la->melody].size(),s.melodies[lb->melody].size());

	bool toptwo=(la->voice!=0 && lb->voice!=0);///can use intervals of a fourth between top two lines
	if (minsize<2 || minsize==melody_length)
		return true;

	M_Line* lowerline;
	M_Line* upperline;

	if (la->voice<lb->voice)
	{
		lowerline=la;
		upperline=lb;
	}
	else if (la->voice>lb->voice)
	{
		upperline=la;
		lowerline=lb;
	}
	else
	{
	//	assert(false);
		upperline=la;
		lowerline=lb;
	}

	Melody uppernotes=absolutify(upperline,s);
	Melody lowernotes=absolutify(lowerline,s);

	int intervalclass=0; //0 dissonant 1 imperfect 2 perfect



	int interval = (uppernotes[minsize-1]-lowernotes[minsize-1]);


	//check for perfect
	for (uint i=0;i<HPINTERVALCOUNT;i++)
	{
		if (interval==h_pintervals[i])
			intervalclass=2;
	}

	//check for imperfect

	for (uint i=0;i<HIPINTERVALCOUNT;i++)
	{
		if (interval==h_ipintervals[i])
			intervalclass=1;
	}

	if (toptwo)
	{
		if (interval==5||interval==12+5||interval==24+5)
			intervalclass=1;
	}

	//get movement type
	int movementtype=0;//0 parallel 1 similar 2 contrary(including oblique)

	//parallel
	if (uppernotes[minsize-1]-uppernotes[minsize-2] == lowernotes[minsize-1]-lowernotes[minsize-2])
	{
		movementtype=0;
	}
	//oblique
	else if (uppernotes[minsize-1]==uppernotes[minsize-2] || lowernotes[minsize-1]==lowernotes[minsize-2])
	{
		movementtype=2;
	}
	//similar
	else if ((uppernotes[minsize-1]<uppernotes[minsize-2]) == (lowernotes[minsize-1]<lowernotes[minsize-2]))
	{
		movementtype=1;
	}
	else//must be contrary?
		movementtype=2;


	return	(intervalclass==1) ||
			(intervalclass==2 && movementtype==2);
}

//allowed melodic intervals
vector<int> candidateSuccessors(Melody* m)
{
	vector<int> r;

	if ( m->size()==0 || m->size() == (melody_length-1))
	{
		r.push_back(5);
	}
	else
	{
		int lastnote=(*m)[m->size()-1];


		for (uint i=0;i<MINTERVALCOUNT;i++)
		{
			int candnote=lastnote+m_intervals[i];
			if (candnote>0 && candnote<=12)
			{
				r.push_back(candnote);
			}
		}
	}


	if (m->size()>1 && m->size()!=(melody_length-1))//can check for tritones
	{
		for (uint i=0; i<r.size();i++)
		{
			int interval=r[i]-(*m)[m->size()-2];
			if (interval==6 || interval==-6 || interval==10 || interval==-10 || interval==11 || interval==-11)
			{ //no tritones or seventh over three notes
				r.erase(r.begin()+i);
				i--;//start again
			}
			else if ((*m)[m->size()-2]==(*m)[m->size()-1] && (*m)[m->size()-1] ==r[i])
			{//no three notes in a row
				r.erase(r.begin()+i);
				i--;//start again
			}
		}
	}
	return r;
}

Melody generateMelody()
{
	Melody m;
	m.clear();
	/*
	for (uint i=0;i<melody_length;i++)
	{
		cout<<"precalc\n";
		vector<int> cands=candidateSuccessors(&m);

		cout<<"assign\n";
		int pick=get_rand()%cands.size();

		cout<<"prepush with pick " << pick << "," << cands[pick] << "\n";
		m.push_back(cands[pick]);

		cout<<"pushback\n";
	}
	cout<<"end generation\n";*/

	return m;
}

Melody generateAloneMelody()
{
	Melody m;

	 for (uint i=0;i<melody_length;i++)
	 {
	 vector<int> cands=candidateSuccessors(&m);

	 int pick=get_rand()%cands.size();

	 m.push_back(cands[pick]);

	 }

	return m;
}



bool imperfect(int a)
{
	for (uint j=0;j<HIPINTERVALCOUNT;j++)
	{
		if (h_ipintervals[j]==a)
			return true;
	}
	return false;
}

bool perfect(int a)
{
	for (uint j=0;j<HPINTERVALCOUNT;j++)
	{
		if (h_pintervals[j]==a)
			return true;
	}
	return false;
}


bool concord(M_Line* la, M_Line* lb, int pos,const backtrackState& s)
{
	Melody a=absolutify(la,s);
	Melody b=absolutify(lb,s);
	assert(a.size()>0&&b.size()>0);
	int i=abs(a[pos]-b[pos]);

	return (perfect(i) ||imperfect(i) || (i==5||i==12+5 ||i==24+5));
}


void testCorrectness(Triangle* t,const backtrackState& s)
{
	t->NoteBad.clear();
	t->anybad=false;

	for (uint i=0;i<melody_length;i++)
	{
		bool foundbad=false;

		for (int j=0;j<3;j++)
			for (int k=j+1;k<3;k++)
			{
				if (!concord(t->line[j], t->line[k], i,s))
				{
					Melody a=absolutify(t->line[j],s);
					Melody b=absolutify(t->line[k],s);
					foundbad=true;
				}
			}

		if (foundbad)
		{
			t->anybad=true;
		}
		t->NoteBad.push_back(foundbad);


	}

}


vector<int> harmonicNotesInRegisterBetween(int a, int b, int position)
{
	vector<int> v;

	//a<b

	for (int i=(position-2)*12;i<=(position-1)*12;i++)
	{
		//imperfect first
		if (imperfect(abs(i-a))&&imperfect(abs(i-b)))
		{
			v.push_back(i-(position-2)*12);
		}
	}

	// if this fails then combination of perfect and imperfect
	if (v.size()==0)
	{
		for (int i=(position-2)*12;i<=(position-1)*12;i++)
		{
			//imperfect first
			if ((imperfect(abs(i-a))&&perfect(abs(i-b))) || (perfect(abs(i-a))&&imperfect(abs(i-b))))
			{
				v.push_back(i-(position-2)*12);
			}
		}
	}


	//then perfect as a last resort
	if (v.size()==0)
	{
		for (int i=(position-2)*12;i<=(position-1)*12;i++)
		{
			if (perfect(abs(i-a))&&perfect(abs(i-b)))
			{
				v.push_back(i-(position-2)*12);
			}
		}
	}

	assert(v.size()>0);
	return v;

}

int commonGoodNote(M_Line* l, M_Line* a=NULL,M_Line*b=NULL );


vector<M_Line*> getConnectedCompleteLines(M_Line* pline);

void generateTriangle(Triangle* t)
{
	/*
	//if blank, generate bass first
	{
		vector<M_Line*> lines = getConnectedCompleteLines(t->line[0]);

		if (t->line[0]->melody->size()==0)
		{

			std::cout<<"GENERATING FIRST MELODY\n";
			std::cout<<"oldmelsize"<<t->line[0]->melody->size()<<std::endl;
			*(t->line[0]->melody)=generateAloneMelody();
			std::cout<<"newmelsize"<<t->line[0]->melody->size()<<std::endl;
		}
	}
	//generate for line 1, if necessary

	if (t->line[1]->melody->size()==0)
	{
		vector<M_Line*> lines = getConnectedCompleteLines(t->line[1]);

		std::cout<<"lastlinesize"<<t->line[0]->melody->size()<<endl;
		assert(lines.size()>0);
		//generate new melody
		for (uint i=0;i<melody_length;i++)
		{
			vector<int> cands=candidateSuccessors(t->line[1]->melody);
			shuffle(cands);

			uint j=0;
			for (j=0;j<cands.size();j++)
			{
				bool allowed=true;

				t->line[1]->melody->push_back(cands[j]);

				for (uint k=0;k<lines.size();k++)
				{
					if (!allowedProgression(t->line[1], lines[k]))
					{
						allowed=false;
						break;
					}
				}

				if (allowed)
				{
					break;
				}
				else
					t->line[1]->melody->pop_back();
			}

			if(j==cands.size())//dead end; will need to thunk something in!
			{

				t->line[1]->melody->push_back(commonGoodNote(t->line[1],t->line[0],t->line[2]));
			}
		}
	}


	//generate for line 1, if necessary

	if (t->line[2]->melody->size()==0)
	{
		vector<M_Line*> lines = getConnectedCompleteLines(t->line[2]);

		assert(lines.size()>0);
		//generate new melody
		for (uint i=0;i<melody_length;i++)
		{
			vector<int> cands=candidateSuccessors(t->line[2]->melody);
			shuffle(cands);

			uint j=0;
			for (j=0;j<cands.size();j++)
			{
				bool allowed=true;

				t->line[2]->melody->push_back(cands[j]);

				for (uint k=0;k<lines.size();k++)
				{
					if (!allowedProgression(t->line[2], lines[k]))
					{
						allowed=false;
						break;
					}
				}

				if (allowed)
				{
					break;
				}
				else
					t->line[2]->melody->pop_back();
			}

			if(j==cands.size())//dead end; will need to thunk something in!
			{
				t->line[2]->melody->push_back(commonGoodNote(t->line[2],t->line[0],t->line[1]));
			}
		}
	}	*/


}

vector<Triangle> triangles;

struct GridTriangle
{
	Triangle* triangle;
	Pos pos;
};

int trianglePop(Triangle* tri)
{
	if (tri==NULL)
		return 0;

	int pop=0;
	for (int i=0;i<3;i++)
		if (tri->line[i]!=NULL)
			pop++;
	return pop;
}


vector<GridTriangle> gridTriangles;

GridTriangle* selectedTriangle;
GridTriangle* hoverTriangle;
GridTriangle* clickTriangle;

struct GridLine
{
	M_Line* line;
	Pos pos;
};

GridLine* selectedLine;
GridLine* hoverLine;
GridLine* clickLine;



bool borderingMelodies(int m1, int m2)
{
	for (uint i=0;i<gridTriangles.size();i++)
	{
		bool found1=false;
		bool found2=false;
		for (uint j=0;j<3;j++)
		{
			if (m1==gridTriangles[i].triangle->line[j]->melody)
				found1=true;
			if (m2==gridTriangles[i].triangle->line[j]->melody)
				found2=true;
		}
		
		if (found1&&found2)
			return true;
	}
	return false;
}

vector<Triangle*> getTriangles(M_Line* pline)
{
	vector<Triangle*> v;
	for (uint i=0;i<gridTriangles.size();i++)
	{
		bool bfound=false;
		for (int j=0;j<3;j++)
		{
			if (gridTriangles[i].triangle->line[j]==pline)
			{
				bfound=true;
				break;
			}
		}
		if (bfound)
			v.push_back(gridTriangles[i].triangle);

	}
	return v;
}


struct StatePosition
{
	int melody;
	int note;
};

struct AlgorithmState
{
	backtrackState s;
	vector<int> choices;
	AlgorithmState():s(backtrackState()),choices(vector<int>())
	{
		choices.clear();
	}
};


int getStatePosition(const backtrackState& s)
{
	int res=-1;

	//if there is an unfinished melody, choose that

	for (uint i=0;i<divineOrdering.size();i++)
	{
		if (s.melodies[divineOrdering[i]].size()>0&&s.melodies[divineOrdering[i]].size()<melody_length)
		{
			res=divineOrdering[i];
		}
	}
	//otherwise pick a new one
	for (uint i=0;i<divineOrdering.size();i++)
	{
		if (s.melodies[divineOrdering[i]].size()==0)
		{
			res=divineOrdering[i];
		}
	}

	assert(res>=0);
	return res;
}

bool algorithmFinished(const backtrackState& s)
{
	for (uint i=0;i<divineOrdering.size();i++)
	{
		if (s.melodies[divineOrdering[i]].size()<melody_length)
		{
			return false;
		}
	}

	return true;
}

M_Line* getM_Line(int melodyIndex)
{
	for (uint i=0;i<m_lines.size();i++)
		if (m_lines[i].melody==melodyIndex)
			return &(m_lines[i]);

	assert(false);
	return NULL;
}

vector<Triangle*> getTriangles(int melodyIndex)
{
	vector<Triangle*> v;
	for (uint i=0;i<gridTriangles.size();i++)
	{
		bool bfound=false;
		for (int j=0;j<3;j++)
		{
			if (gridTriangles[i].triangle->line[j]->melody==melodyIndex)
			{
				bfound=true;
				break;
			}
		}
		if (bfound)
			v.push_back(gridTriangles[i].triangle);
	}
	return v;
}

int getMelodyVoice(int melodyIndex)
{
	for (uint i=0;i<m_lines.size();i++)
	{
		if (m_lines[i].melody==melodyIndex)
		{
			return m_lines[i].voice;
		}
	}
	assert(false);
}

vector<int> getPermittedNotes(backtrackState& s)
{
	vector<int> v;

	int curmelody=getStatePosition(s);

	//find candidate melodic successors
	vector<int> candidateMelodicSuccessors = candidateSuccessors(&(s.melodies[curmelody]));

	//find which of these satisfies all of the harmonic constraints
	vector<Triangle*> tris = getTriangles(curmelody);

	//get all full melodies
	set<int> countermelodies;
	for (uint i=0;i<tris.size();i++)
	{
		for (uint j=0;j<3;j++)
		{
			if (s.melodies[tris[i]->line[j]->melody].size()==melody_length)
			{
				countermelodies.insert(tris[i]->line[j]->melody);
			}
		}
	}

	//for each note, test against every melody
	for (uint i=0;i<candidateMelodicSuccessors.size();i++)
	{
		bool ballowed=true;

		s.melodies[curmelody].push_back(candidateMelodicSuccessors[i]);
		for (set<int>::iterator j=countermelodies.begin();j!=countermelodies.end();j++)
		{
			if (!allowedProgression(getM_Line(curmelody), getM_Line(*j), s))
			{
				ballowed=false;
				break;
			}
		}
		s.melodies[curmelody].pop_back();

		if(ballowed)
		{
			v.push_back(candidateMelodicSuccessors[i]);
		}

	}

	//dont forget to shuffle vector before returning!
	shuffle(v);
	return v;
}





backtrackState chooseParameter(AlgorithmState& oldState)
{
	backtrackState newBtState=oldState.s;
	int curmelody=getStatePosition(newBtState);
	newBtState.melodies[curmelody].push_back(oldState.choices.back());

	oldState.choices.pop_back();
	return newBtState;

}

backtrackState backtrack(uint seed)
{
	init_rand(seed);

	vector<AlgorithmState> history;

	//initialize the algorithm
	AlgorithmState initState;

	initState.choices=getPermittedNotes(initState.s);

	history.push_back(initState);

	int iterations=0;

	AlgorithmState newState;

	while (true)
	{

		if (history.back().choices.size()==0)
		{
			//get position
			int concernedmelody=getStatePosition(history.back().s);
			int concernedposition=history.back().s.melodies[concernedmelody].size();
			
			int curmel=0;
			int curpos=0;
			while ( history.back().choices.size()==0 || (!borderingMelodies(concernedmelody,curmel)) || (curpos>concernedposition))
			{
				assert(history.size()>0);
				history.pop_back();
				curmel=getStatePosition(history.back().s);
				curpos=history.back().s.melodies[concernedmelody].size();

			}	
		}

		assert(history.size()>0);

		newState.s = chooseParameter(history.back());


		if (algorithmFinished(newState.s))
		{
			return newState.s;
		}

		newState.choices = getPermittedNotes(newState.s);

		history.push_back(newState);

		if (algorithmFinished(newState.s))
		{
			return newState.s;
		}


		if (key[KEY_ESC])
		{
			quit=true;
			return newState.s;
		}

		iterations++;
		
		if (iterations%500==0)
		{
		clear_to_color(buffer, bgcol);
		sprintf(leveltitle, "level code: %d",levelseed);
		textout_ex(buffer, font, leveltitle, 100, 60, white, -1);

		textout_ex(buffer, font, "generating level", 100, 80, white, -1);
		sprintf(leveltitle, "search depth: %d", history.size());
		textout_ex(buffer, font, leveltitle, 100, 100, white, -1);
		sprintf(leveltitle, "iterations: %d", iterations);
		textout_ex(buffer, font, leveltitle, 100, 120, white, -1);
		for (uint i=0;i<history.size();i++)
		{
			hline(buffer, 4, i, 4+history[i].choices.size(), black);
		}

		stretch_blit(buffer, screen, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, 0, 640, 480);
		}
	}
}

int commonGoodNote(M_Line* l, M_Line* ma,M_Line*mb,const backtrackState& s )
{
	int position=s.melodies[l->melody].size();

	vector<Triangle*> tris=getTriangles(l);


	vector< vector<int> > vs;

	for (uint i=0;i<tris.size();i++)
	{
		vector <Melody> a;
		for (int k=0;k<3;k++)
		{
			if (l!=tris[i]->line[k] && s.melodies[tris[i]->line[k]->melody].size()==melody_length)
			{

				a.push_back(absolutify(tris[i]->line[k],s));
			}
		}
		vs.push_back(harmonicNotesInRegisterBetween(a[0][position],a[1][position],l->voice));
	}

	if (ma&&mb&&s.melodies[ma->melody].size()==melody_length&&s.melodies[mb->melody].size()==melody_length)
	{

		vs.push_back(harmonicNotesInRegisterBetween(s.melodies[ma->melody].at(position),s.melodies[mb->melody].at(position),l->voice));

	}

	assert(vs.size()>0);

	
	//now to intersect all with the first
	for (uint j=0;j<vs[0].size();j++)
	{//for each slice
		for (uint i=1;i<vs.size();i++)
		{

			bool found=false;
			for (uint k=0;k<vs[i].size();k++)
			{
				if (vs[i][k]==vs[0][j])
				{
					found=true;
					break;
				}
			}
			if (!found)
			{
				vs[0].erase(vs[0].begin()+j);
				j--;
			}
		}


	}


	assert(vs[0].size()>0);
	return vs[0][get_rand()%vs.size()];
}


vector<M_Line*> getConnectedCompleteLines(M_Line* pline,const backtrackState& s)
{
	vector<Triangle*> tris = getTriangles(pline);

	vector<M_Line*> lines;

	for (uint i=0;i<tris.size();i++)
	{
		for (int j=0;j<3;j++)
		{
			bool found=false;
			for (uint k=0;k<lines.size();k++)
			{
				if (tris[i]->line[j]==lines[k])
				{
					found=true;
					break;
				}
			}

			if(!found && (s.melodies[tris[i]->line[j]->melody].size()==melody_length ) && tris[i]->line[j]!=pline )
				lines.push_back(tris[i]->line[j]);
		}
	}

	return lines;
}

vector<GridLine> gridLines;


void swapLines(GridLine* a, GridLine* b)
{
	swap(a->line->melody,b->line->melody);
	swap(a->line->voice,b->line->voice);
}

void shuffleLines()
{
	int n=gridLines.size();
	while(n>1)
	{
		n--;
		int k = get_rand()%(n+1);
		swapLines(&(gridLines[k]),&(gridLines[n]));
	}
}





bool inSelectedTri(const GridLine* gridLine)
{
	if (selectedTriangle==NULL)
		return false;

	for(uint i=0;i<3;i++)
	{
		if (selectedTriangle->triangle->line[i]->melody==gridLine->line->melody)
			return true;
	}

	return false;
}



int target[2];
bool inbase[2];

Pos pos[2];
Pos base[2];

const int CIRCLECOUNT=20;

Pos circles[CIRCLECOUNT];
Pos circelvels[CIRCLECOUNT];

//used for targetting numerical enemies




void drawTriangle (const GridTriangle* t)
{
	if (t==selectedTriangle)
		circlefill(buffer, t->pos.x, t->pos.y, ICON_RADIUS,  yellow );

	int col=lightgrey;
	int col2=lightgrey;

	if ( selectedTriangle==t)
	{
		if (curnote<melody_length )
		{
			if (t->triangle->NoteBad[curnote])
			{
				col=red;
			}
			else
			{
				col=green;
			}
		}

		if (t->triangle->anybad)
		{
			col2=red;
		}
		else
		{
			col2=green;
		}
	}
	else
	{
		if (t->triangle->anybad)
		{
			col2=lightred;
		}
		else
		{
			col2=lightgreen;
		}
	}


	if (selectedTriangle==t)
	{
		circlefill(buffer, t->pos.x, t->pos.y, ICON_RADIUS, col );
	}

	circle(buffer, t->pos.x, t->pos.y, ICON_RADIUS, col2 );
}

char name[2]="a";

bool isSelectedLine(const GridLine* l)
{
	if ( l==selectedLine )
		return true;

	if (selectedTriangle!=NULL)
	{
		if (selectedTriangle->triangle!=NULL)
		{
			for (int i=0;i<3;i++)
			{
				if (l->line==selectedTriangle->triangle->line[i])
					return true;
			}
		}
	}

	return false;
}

void drawLine (const GridLine* l)
{
	int col=green;

	if (l!=NULL&&l->line!=NULL)
	{
		if (l->line->voice==0)
			col=purp0;
		else if (l->line->voice==1)
			col=purp1;
		else if (l->line->voice==2)
			col=purp2;
	}
	if (isSelectedLine(l))
		circlefill(buffer,l->pos.x,l->pos.y,ICON_RADIUS,col);
	else
		circle(buffer, l->pos.x, l->pos.y, ICON_RADIUS,col );


	int i=l->line->melody;
	if (!(dragmode&&clickLine==l))
	{
		name[0]='A'+i;
		textout_centre_ex(buffer, font, name, l->pos.x, l->pos.y-FONT_HEIGHT/2, black, -1);
	}
}

#include "sound.hpp"
#include "timer.hpp"

const int ts=13;
bool leave=false;
int titletimer=0;
bool finishedtitle;

bool escpressed;
bool apressed;

bool collided;


void cutbars()
{
	rectfill(buffer, 0, 0, SCREEN_WIDTH, 2, black);
	rectfill(buffer, 0, SCREEN_HEIGHT-3, SCREEN_WIDTH, SCREEN_HEIGHT, black);
}

struct State
{
	int x;
	int frame;
};

State wife;
State husband;
State daughter;

int leveltimer;

int levelgoal;

bool stoppedfather;

void onSelect()
{
	musictimer=(melody_length+3)*FRAMERATE;
	curnote=musictimer/FRAMERATE;
	stopnotes();
}

void onClick()
{
	dragmode=false;
	clickPos=mouse;
	clickLine=hoverLine;
	clickTriangle=hoverTriangle;
}

void onMouseUp()
{

	if (dragmode==false)
	{
		if (clickLine)
		{
			selectedLine=clickLine;
			selectedTriangle=NULL;
			onSelect();
		}
		else if (clickTriangle)
		{
			selectedTriangle=clickTriangle;
			selectedLine=NULL;
			onSelect();
		}
		else
		{
			selectedTriangle=NULL;
			selectedLine=NULL;
			onSelect();
		}
	}
	else
	{
		if (clickLine&&hoverLine)
		{
			swapLines(clickLine, hoverLine);

			vector<Triangle*> v1=getTriangles(clickLine->line);
			for (uint i=0;i<v1.size();i++)
			{
				testCorrectness(v1[i],gameMelodies_State);
			}
			vector<Triangle*> v2=getTriangles(hoverLine->line);

			for (uint i=0;i<v2.size();i++)
			{
				testCorrectness(v2[i],gameMelodies_State);
			}

			bool won=true;
			for (uint i=0;i<triangles.size();i++)
			{
				if (triangles[i].anybad)
				{
					won=false;
					break;
				}
			}
			if (won&&gridLines.size()>3)
			{
				gameover=true;
			}
			else
			    gameover=false;

			if (isSelectedLine(clickLine) ||isSelectedLine(hoverLine))
				onSelect();
		}

		dragmode=false;
	}
}

void onDrag()
{
	if (distsq(clickPos, mouse)>ICON_RADIUS*ICON_RADIUS/4)
	{
		dragmode=true;
	}
}

M_Line* getLine(Pos p,int voiceno)
{
	for (uint i=0;i<gridLines.size();i++)
	{
		if (distsq(p, gridLines[i].pos)<ICON_RADIUS*ICON_RADIUS*3*4)
		{
			if (gridLines[i].line->voice==voiceno)
				return gridLines[i].line;
		}
	}

	assert(false);
	return NULL;

}

bool divinated(int n)
{
	for (uint i=0;i<divineOrdering.size();i++)
	{
		if (n==divineOrdering[i])
			return true;
	}
	return false;
}

/*
bool addToDivineOrdering()
{
	//look for a triangle with two filled and one free
	for (uint i=0;i<triangles.size();i++)
	{
		int divinecount=0;
		int free=-1;
		int nope=false;;

		for (int j=0;j<3;j++)
			if (triangles[i].line[j]==NULL)
				nope=true;
		if (nope)
		{
			cout<<"NOPE\n";
			break;
		}

		for (int j=0;j<3;j++)
		{
			if (divinated(triangles[i].line[j]->melody))
			{
				divinecount++;
			}
			else
				free=triangles[i].line[j]->melody;
		}
		if (divinecount==2)
		{
			cout<<"AFREE"<<free<<endl;
			divineOrdering.push_back(free);
			return true;
		}
	}

	//else if you can find a piece with one neighbour, add that
	//look for a triangle with two filled and one free
	for (uint i=0;i<triangles.size();i++)
	{
		int divinecount=0;
		int free=-1;
		int nope=false;;

		for (int j=0;j<3;j++)
			if (triangles[i].line[j]==NULL)
				nope=true;
		if (nope)
		{
			cout<<"NOPE\n";
			break;
		}

		for (int j=0;j<3;j++)
		{
			if (divinated(triangles[i].line[j]->melody))
			{
				divinecount++;
			}
			else
				free=(triangles[i].line)[j]->melody;
		}
		if (divinecount==1)
		{
			cout<<"BFREE"<<free<<endl;
			divineOrdering.push_back(free);
			return true;
		}
	}

	//else if you can find a piece with no neighbours, add that

	//look for a triangle with two filled and one free
	for (uint i=0;i<triangles.size();i++)
	{
		int nope=false;

		for (int j=0;j<3;j++)
			if (triangles[i].line[j]==NULL)
				nope=true;
		if (nope)
		{
			cout<<"CNOPE\n";
			break;
		}

		int divinecount=0;
		for (int j=0;j<3;j++)
		{
			if (divinated(triangles[i].line[j]->melody))
			{
				divinecount++;
			}
		}
		if (divinecount==0)
		{
			divineOrdering.push_back(triangles[i].line[0]->melody);
			return true;
		}
	}

	//else done
	return false;
}*/

void calculateDivineOrdering()
{
	divineOrdering.clear();
	for (uint i=0;i<30;i++)
	{
		divineOrdering.push_back(orderchars[i]-'A');
	}
//	while (addToDivineOrdering()){};
}


class LevelLoop
	{
	public:
		static void initialize()
		{
			algdone=false;
			gameover=false;
			init_rand(levelseed);
			get_rand();
			level_code=get_rand();
			selectedLine=NULL;
			hoverLine=NULL;
			clickLine=NULL;
			hoverTriangle=NULL;
			clickTriangle=NULL;
			selectedTriangle=NULL;
			hoverLine=NULL;
			dragmode=false;


			mb1=false;
			mb2=false;

			highlight.x=-1;//means not selected
			melody_length=7+(get_rand()%4);
			escpressed=true;
			quit=false;
			finishedtitle=false;


			gameMelodies_State.melodies.clear();

			triangles.clear();
			m_lines.clear();
			gridTriangles.clear();
			gridLines.clear();

			gameMelodies_State.melodies.reserve(200);
			triangles.reserve(200);
			m_lines.reserve(200);
			gridTriangles.reserve(200);
			gridLines.reserve(200);


			bool indent=false;

			int voice=0;

			//make all the lines
			//1 draw grid outline (could buffer to separate bitmap at init actually)
			for (double j=MARGIN+ICON_RADIUS;j<SCREEN_HEIGHT-ICON_RADIUS;j+=(sqrt(3)*TRIANGLE_SIZE)/2)
			{
				if (indent)
					voice=0;
				else
					voice=2;

				for (float i=MARGIN+ICON_RADIUS;i<SCREEN_WIDTH-MARGIN-ICON_RADIUS;i+=TRIANGLE_SIZE)
				{
					Melody m=generateMelody();
					gameMelodies_State.melodies.push_back(m);
					gameMelodies_State.melodies[gameMelodies_State.melodies.size()-1].reserve(melody_length+1);

					M_Line m_line;
					m_line.melody=gameMelodies_State.melodies.size()-1;
					m_line.voice=voice;
					m_lines.push_back(m_line);

					GridLine gl;
					gl.pos.x=i+(indent?TRIANGLE_SIZE/2:0);
					gl.pos.y=j;
					gl.line=&(m_lines[m_lines.size()-1]);
					gridLines.push_back(gl);
					voice=(voice+2)%3;
				}
				indent=!indent;
			}


			int trianglecount=0;
			//make all the triangles

			clear_to_color(buffer, bgcol);
			textout_ex(buffer, font, "generating triangles", 100, 100, white, -1);
			stretch_blit(buffer, screen, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, 0, 640, 480);


			for (int j=MARGIN+ICON_RADIUS+(sqrt(3)*TRIANGLE_SIZE/6);j<SCREEN_HEIGHT-ICON_RADIUS-MARGIN-TRIANGLE_SIZE;j+=(sqrt(3)*TRIANGLE_SIZE)/2)
			{
				bool indenty=true;
				for (int i=MARGIN+ICON_RADIUS+TRIANGLE_SIZE/2;i<SCREEN_WIDTH-ICON_RADIUS-MARGIN;i+=TRIANGLE_SIZE/2)
				{


					GridTriangle gt;
					gt.pos.x=i-(indent?TRIANGLE_SIZE/2:0);
					if (indenty)
						gt.pos.y=j-2*(sqrt(3)*TRIANGLE_SIZE/6)+(sqrt(3)*TRIANGLE_SIZE/2);
					else
						gt.pos.y=j;



					if (gt.pos.x>TRIANGLE_SIZE/3+ICON_RADIUS
						&& gt.pos.x<SCREEN_WIDTH-TRIANGLE_SIZE/2-MARGIN && gt.pos.y<SCREEN_HEIGHT-TRIANGLE_SIZE/3)
					{
						Triangle t;
						t.line[0]=getLine(gt.pos, 0);
						t.line[1]=getLine(gt.pos, 1);
						t.line[2]=getLine(gt.pos, 2);

						if (get_rand()%10==0)
						{
							triangles.push_back(t);

							gt.triangle=&(triangles[triangles.size()-1]); ///MUST CHANGE (why?)
							gridTriangles.push_back(gt);
						}

						//cout<<"beginning function tri\n";
						//generateTriangle(gridTriangles[gridTriangles.size()-1].triangle);
						//cout<<"ending function tri\n";
						trianglecount++;


					}
					indenty=!indenty;
				}
				indent=!indent;
			}

			//remove unnecessary melodies

			for (uint i=0;i<gridLines.size();i++)
			{
				if (getTriangles(gridLines[i].line->melody).size()==0)
				{
					gridLines.erase(gridLines.begin()+i);
					i=0;
				}
			}

			calculateDivineOrdering();
			gameMelodies_State=backtrack(levelseed);

			if (quit) return;

			clear_to_color(buffer, bgcol);
			textout_ex(buffer, font, "shuffling", 70, 100, white, -1);
			stretch_blit(buffer, screen, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, 0, 640, 480);

			shuffleLines();


			clear_to_color(buffer, bgcol);
			textout_ex(buffer, font, "calculating corectnesses", 70, 100, white, -1);
			stretch_blit(buffer, screen, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, 0, 640, 480);

			for (uint i=0;i<gridTriangles.size();i++)
			{
				testCorrectness(gridTriangles[i].triangle,gameMelodies_State);
			}

			if (gridLines.size()==3)
				sprintf(leveltitle, "level code: %d [asteroid]",levelseed);
			else if (gridTriangles.size()==1)
				sprintf(leveltitle, "level code: %d [orbital]",levelseed);
			else if (gridTriangles.size()==0)
				sprintf(leveltitle, "level code: %d [void]",levelseed);
			else
				sprintf(leveltitle, "level code: %d",levelseed);

		}
		static bool quitcondition()
		{
			return quit||finishedtitle;
		}
		static void logic()
		{

			if (selectedLine || selectedTriangle)
			{
				musictimer=(musictimer+1) % ((melody_length+4)*FRAMERATE);

				curnote=musictimer/FRAMERATE;
				if (musictimer%FRAMERATE==0)
				{
					uint note=musictimer/FRAMERATE;
					if (note<melody_length)
					{
						if (selectedTriangle)
						{
							for (int i=0;i<3;i++)
							{
								playnote((selectedTriangle->triangle->line[i]->voice-2)*12+gameMelodies_State.melodies[selectedTriangle->triangle->line[i]->melody][note]);
							}
						}
						else if (selectedLine)
						{
							playnote((selectedLine->line->voice-2)*12+gameMelodies_State.melodies[selectedLine->line->melody][note]);
						}
					}
				}

			}
			else
			{
				curnote=melody_length+40;
			}



			if (key[KEY_ESC]&&!escpressed)
			{
				quit=true;
			}
			escpressed=key[KEY_ESC];

			mouse.x=(mouse_x*SCREEN_WIDTH)/640;
			mouse.y=(mouse_y*SCREEN_HEIGHT)/480;

			hoverTriangle=NULL;
			hoverLine=NULL;
			highlight.x=-1;

			//check triangles first
			if (! (dragmode&&clickLine))
			{
				for (uint i=0;i<gridTriangles.size();i++)
				{
					if (distsq(mouse, gridTriangles[i].pos)<=ICON_RADIUS*ICON_RADIUS+2)
					{
						hoverTriangle=&(gridTriangles[i]);
						highlight=gridTriangles[i].pos;
						break;
					}
				}
			}

			//then check lines
			if (highlight.x==-1)
			{
				for (uint i=0;i<gridLines.size();i++)
				{
					if (distsq(mouse, gridLines[i].pos)<=ICON_RADIUS*ICON_RADIUS+2)
					{
						hoverLine=&(gridLines[i]);
						highlight=gridLines[i].pos;
						break;
					}
				}
			}

			bool mb1new = mouse_b & 1;
			bool mb2new = mouse_b & 2;

			if (mb1new&&!mb1)
				onClick();
			else if(mb1&&!mb1new)
				onMouseUp();
			else if (mb1)
				onDrag();

			mb1=mb1new;
			mb2=mb2new;
		}
		static void draw()
		{


			clear_to_color(buffer,bgcol);

			for (uint i=0;i<gridTriangles.size();i++)
				drawTriangle(&(gridTriangles[i]));

			for (uint i=0;i<gridLines.size();i++)
				drawLine(&(gridLines[i]));

			if (highlight.x!=-1)
			{
				circle(buffer, highlight.x, highlight.y, ICON_RADIUS+3, white);
			}

			textout_ex(buffer, font, leveltitle, MARGIN+1, 220, black, -1);

            if (gameover)
			{
			 			textout_centre_ex(buffer, font, "and you won", 160, 4*10,white, -1);
		            	textout_centre_ex(buffer, font, "and nothing will be the same again", 160, 5*10,white, -1);
			}

			//draw cursor
			{
				if (dragmode&&clickLine)
				{
					name[0]='A'+clickLine->line->melody;

					textout_centre_ex(buffer, font, name, mouse.x-4, mouse.y+6-FONT_HEIGHT/2, black, -1);
				}

				rectfill(buffer, mouse.x-2, mouse.y-2, mouse.x+2, mouse.y+2, black);
			}


		}
	};

template<int i>
class BlackLoop
	{
	public:
		static void initialize()
		{
			escpressed=true;
			bltime=0;
			finishedtitle=false;
		}
		static bool quitcondition()
		{
			return quit||finishedtitle;
		}
		static void logic()
		{
			bltime++;
			if (bltime>i*FRAMERATE)
			{
				finishedtitle=true;
			}


			if (key[KEY_ESC]&&!escpressed)
			{
				quit=true;
			}
			escpressed=key[KEY_ESC];

		}
		static void draw()
		{
			clear_to_color(buffer, bgcol);
		}
	};


template<class T> void loop();



bool spacepressed;
int storyprogress;
bool finishedstory;
int pauseafter;



class IntroLoop
	{
	public:
		static void initialize()
		{
			spacepressed=true;
			escpressed=true;
			speed_count=0;
			titletimer=0;
			finishedtitle=false;
		}

		static bool quitcondition()
		{
			return quit||finishedtitle;
		}

		static void logic()
		{
			titletimer++;

			if (titletimer>120||(mouse_b&1)||keypressed())
			{
				if (titletimer<=120)
					playnote(rand()%24 - 12);

				position_mouse(400,200);
				finishedtitle=true;
			}
		}

		static void draw()
		{
		    clear_to_color(buffer,bgcol);
			textout_centre_ex(buffer, font, "increpare presents", 160, 4*10,white, -1);

		}
	};


class GameOverLoop
	{
	public:
		static void initialize()
		{

			spacepressed=true;
			quit=false;
			escpressed=true;
			speed_count=0;
			titletimer=0;
			finishedtitle=false;
		}

		static bool quitcondition()
		{
			return quit||finishedtitle;
		}

		static void logic()
		{


			if ((key[KEY_ESC]&&!escpressed)||key[KEY_SPACE])
			{
				quit=true;
			}
			escpressed=key[KEY_ESC];

		}

		static void draw()
		{
		    clear_to_color(buffer,bgcol);
			textout_centre_ex(buffer, font, "and you won", 160, 4*10,white, -1);
			textout_centre_ex(buffer, font, "and nothing will be the same again", 160, 5*10,white, -1);
		}
	};


class TitleLoop
	{
	public:
		static bool begin;
		static unsigned int selected;
		static bool upheld;
		static bool downheld;
		static bool spaceheld;
	public:
		static void initialize()
		{
			levelseed=1;
			spacepressed=true;
			endmode=false;
			endmenutimer=0;
			quit=false;
			begin=false;
			selected=0;
			upheld=true;
			downheld=true;
			spaceheld=true;
			escpressed=true;

		}

		static void logic()
		{

			if (endmenutimer>0)
			{
				endmenutimer--;


				return;
			}

			if (key[KEY_ESC]&&!escpressed)
			{
				quit=true;
			}
			escpressed=key[KEY_ESC];



			if (key[KEY_UP])
			{
				if (!upheld)
				{
					selected=(selected+1)%2;
					playnote(rand()%24 - 12);
					upheld=true;
				}
			}
			else
				upheld=false;

			if (key[KEY_DOWN])
			{
				if (!downheld)
				{
					selected=(selected+1)%2;
					playnote(rand()%24 - 12);
					downheld=true;
				}
			}
			else
				downheld=false;

			if (key[KEY_SPACE])
			{
				if (!spaceheld)
				{
					if (selected==0)
					{
						begin=true;
						endmenutimer=40;
						playnote(rand()%24 - 12);
					}
					else
					{
						quit=true;
						playnote(rand()%24 - 12);
						endmenutimer=80;
					}
				}
			}
			else
				spaceheld=false;

		}

		static bool quitcondition()
		{
			return (quit||begin) && endmenutimer==0;
		}

		static void draw()
		{
            clear_to_color(buffer,bgcol);

			if ( (endmenutimer>0 && endmenutimer<10))
			{
				return;
			}

			textout_ex(buffer,font,"concord",2*79+60,2*5,makecol(255,255,255),-1);

			if (selected==0)
			{
				if (endmenutimer%2==0)
				textout_ex(buffer,font," START ",60+50,120,bgcol,makecol(255,255,255));
				textout_ex(buffer,font," Exit ",60+50,140,makecol(255,255,255),-1);
			}
			else
			{
				textout_ex(buffer,font," Start ",60+50,120,makecol(255,255,255),-1);

				if (endmenutimer%2==0)
				textout_ex(buffer,font," EXIT ",60+50,140,bgcol,makecol(255,255,255));
			}

		}

	};

bool TitleLoop::begin=false;
unsigned int TitleLoop::selected=0;
bool TitleLoop::upheld=false;
bool TitleLoop::downheld=false;
bool TitleLoop::spaceheld=true;


class SelectLoop
	{
	public:
		static bool begin;
		static unsigned int selected;
		static bool upheld;
		static bool downheld;
		static bool spaceheld;
	public:
		static void initialize()
		{

			spacepressed=true;
			endmode=false;
			endmenutimer=0;
			quit=false;
			begin=false;
			selected=0;
			upheld=true;
			downheld=true;
			spaceheld=true;
			escpressed=true;

		}

		static void logic()
		{

			if (endmenutimer>0)
			{
				endmenutimer--;


				return;
			}

			if (key[KEY_ESC]&&!escpressed)
			{
				quit=true;
			}
			escpressed=key[KEY_ESC];



			if (key[KEY_UP])
			{
				if (!upheld)
				{
					levelseed--;
					playnote(rand()%24 - 12);
					upheld=true;
				}
			}
			else
				upheld=false;

			if (key[KEY_DOWN])
			{
				if (!downheld)
				{
					levelseed++;
					playnote(rand()%24 - 12);
					downheld=true;
				}
			}
			else
				downheld=false;

			if (key[KEY_SPACE])
			{
				if (!spaceheld)
				{
					if (selected==0)
					{
						begin=true;
						endmenutimer=40;
						playnote(rand()%24 - 12);
					}

				}
			}
			else
				spaceheld=false;

		}

		static bool quitcondition()
		{
			return (quit||begin) && endmenutimer==0;
		}

		static void draw()
		{
            clear_to_color(buffer,bgcol);

			if ( (endmenutimer>0 && endmenutimer<10))
			{
				return;
			}

			textout_ex(buffer,font,"concord",2*79+50,2*5,makecol(255,255,255),-1);

			textout_ex(buffer,font," choose seed ",60,2*30+40,white,-1);

			sprintf(leveltitle, "%d",levelseed-1);
			textout_centre_ex(buffer,font,leveltitle,120,140,white,-1);
			sprintf(leveltitle, "<%d>",levelseed);
			if (endmenutimer%2==0)
				textout_centre_ex(buffer,font,leveltitle,120,150,bgcol,makecol(255,255,255));
			sprintf(leveltitle, "%d",levelseed+1);
			textout_centre_ex(buffer,font,leveltitle,120,160,white,-1);


		}

	};

bool SelectLoop::begin=false;
unsigned int SelectLoop::selected=0;
bool SelectLoop::upheld=false;
bool SelectLoop::downheld=false;
bool SelectLoop::spaceheld=true;

int endtimer=0;

void winlevel()
{
	if (endtimer>0)
		return;

	endtimer=120;

}

void loselevel()
{
	endtimer=120;
}



template<class T>
void loop()
{

	clear_to_color(buffer, bgcol);

	T::initialize();

	while(!T::quitcondition())
	{
		al_poll_duh(dplay);

		while(speed_count>0)
		{
			poll_mouse();

			//			if (key[KEY_ESC])
			//				quit=true;

			T::logic();

			speed_count--;

		}

		T::draw();

        stretch_blit(buffer, screen, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, 0, 640, 480);

	}
}


void init()
{

	/* initialise everything */
	allegro_init();
	init_sound();

	set_color_depth(32);

	install_keyboard();
	install_mouse();
	install_timer();


	#ifdef __APPLE__
	CGDisplayHideCursor(kCGDirectMainDisplay);
	#endif

	set_alpha_blender();

    LOCK_VARIABLE(speed_counter); //Used to set the timer - which regulates the game's
    LOCK_FUNCTION(increment_speed_counter);//speed.
    install_int_ex(increment_speed_counter, BPS_TO_TIMER(FRAMERATE));

	set_gfx_mode(GFX_AUTODETECT_FULLSCREEN, 640, 480, 0, 0) ;

	red=makecol(255,0,0);
	lightred=makecol(255,200,200);
	magenta=makecol(255,0,255);
	black=makecol(0,0,0);
	white=makecol(255,255,255);
	blue=makecol(0,0,255);
	bgcol=makecol(193,185,220);
	yellow=makecol(255,255,0);
	green=makecol(0,255,0);
	lightgreen=makecol(200,255,200);
	grey=makecol(100,100,100);
	lightgrey=makecol(200,200,200);
	orange=makecol(253,183,58);
	purp0=makecol(107,30,98);
	purp1=makecol(199,40,162);
	purp2=makecol(255,220,255);

	//load save data


	//init the buffer
	buffer=create_bitmap(SCREEN_WIDTH, SCREEN_HEIGHT);

}

int dir;
int px;
int py;

bool gototitle=false;

int main(int argc, char *argv[])
{

	init();


	loop<IntroLoop>();

titletag:
	gototitle=false;

	loop<TitleLoop>();

	if (quit)
		goto end;

selecttag:

	loop<SelectLoop>();

	if (quit)
	{
		gototitle=true;
		goto menuskip;
	}

	loop<LevelLoop >();



menuskip:
	quit=false;
	finishedtitle=false;
	TitleLoop::begin=false;
	if (gototitle)
		goto titletag;
	if (gameover)
		goto selecttag;
	goto selecttag;

end:
	quit=false;
	return 0;

}
END_OF_MAIN()
